# Copyright (c) 2023 PYCAD
# This file is part of the PYCAD library and is released under the MIT License:
# https://github.com/amine0110/pycad/blob/main/LICENSE


from .pancreas_dataset import PancreasDataset
from .hippocampus_dataset import HippocampusDataset
from .heart_dataset import HeartDataset
from .brain_tumor_dataset import BrainTumorDataset
from .colon_dataset import ColonDataset
from .spleen_dataset import SpleenDataset
from .lung_tumor_dataset import LungTumorDataset
from .prostate_dataset import ProstateDataset
from .liver_and_tumor_dataset import LiverAndTumorDataset